@@ -1,32 +1,49 @@
+#include <config.h>
+
+
 /* If we use history databases, then these routines will */
 /* get compiled and used.  If we don't, the routines in chkhistory.c will */
 /* be used. */
 
 #include <stdio.h>
-#include <netdb.h>
-#include "config.h"
+#include "suck_config.h"
 #include "suck.h"
 #include "both.h"
 #include "chkhistory.h"
+#include "suckutils.h"
 
 #ifdef TIMER
 #include "timer.h"
 #endif
+
+/* These take care if user had multiple defines in makefile */
+/* use DBM if we've got it */
 #ifdef USE_DBM
-#	include <dbm.h>
-#	define close_history() dbmclose();
+#undef USE_NDBM
+#undef USE_DBZ
+#endif
+/* else if have NDBM use it */
+#ifdef USE_NDBM
+#undef USE_DBZ
+#endif
+
+#ifdef USE_DBM
+#include <dbm.h>
+#define close_history() dbmclose();
 #endif
 
 #ifdef USE_NDBM
-#	include <ndbm.h>
-#	include <fcntl.h>
-	static DBM *db = NULL;	/* I know this isn't too pretty, but its the easiest way to do it */
-#	define close_history() dbm_close(db)
+#include <ndbm.h>
+#ifdef HAVE_FCNTL_H
+#include <fcntl.h>
+#endif
+static DBM *db = NULL;	/* I know this isn't too pretty, but its the easiest way to do it */
+#define close_history() dbm_close(db)
 #endif
 
 #ifdef USE_DBZ
-#	include <dbz.h>
-#	define close_history() dbmclose()
+#include <dbz.h>
+#define close_history() dbmclose()
 #endif
 
 /* function prototypes */
@@ -43,7 +60,8 @@ void chkhistory(PMaster master) {
 	}
 	else {
 
-		fprintf(master->msgs, "Processing History Database\n");
+		fprintf(master->msgs, "Processing History Database ");
+		fflush(master->msgs);	/* so msg gets printed */
 #ifdef TIMER
 		TimerFunc(TIMER_START, 0L);
 #endif
