@@ -1,11 +1,19 @@
+#include <config.h>
+
 #include <stdio.h>
 #include <stdlib.h>
-#include <netdb.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 #include <string.h>
 #include <ctype.h>
 
-#include "config.h"
+#include "suck_config.h"
+
+#ifdef USE_REGEX
+#include <regex.h>
+#endif
+
 #include "both.h"
 #include "suck.h"
 #include "suckutils.h"
@@ -16,7 +24,8 @@
 
 #define PATHHOST_SEPARATOR ','		/* character used to separate hosts in the pathhost line in killfile */
 #define SUBJECT_SEPARATOR ','		/* character used to separate words in the subject line in killfile */
-#define FROM_SEPARATOR ','		/* character used to separate names in the subject line in killfile */
+#define FROM_SEPARATOR ','		/* character used to separate words in the subject line in killfile */
+#define NNTP_SEPARATOR ','		/* character used to separate names in the nntphost line in killfile */
 
 #define GROUP_KEEP "keep"
 #define GROUP_DELETE "delete"		/* word in param file parsed for on group line to signify keep or delete */
@@ -24,14 +33,19 @@
 
 /* local function prototypes */
 char *get_a_chunk_mem(PMaster);
-int chk_msg_kill(PKillStruct, char *);
-char *malloc_line(int which, char *linein);
-int check_line(char *, char *, char *, int, int, char);
+int check_line(char *, const char *, const char *, int, int, char);
 void free_node(OneKill);
-int parse_a_file(char *, char *, POneKill);
+int parse_a_file(const char *, const char *, POneKill);
 int pass_two(PKillStruct);
 int check_a_group(POneKill, char *, int *);
-char *strnstr(char *, char *);
+const char *strnstr(const char *, const char *);
+
+#ifdef USE_REGEX
+void regex_scan(pmy_regex, char *, char, char);
+int regex_check(char *, const char *, pmy_regex);
+#else
+char *malloc_line(int which, const char *linein);
+#endif
 /*-------------------------------------------------------------------------*/
 const char *Reasons[] = {
 	"No Reason - major screwup",
@@ -40,18 +54,20 @@ const char *Reasons[] = {
 	"Host in Path rejected",
 	"Word in Subject rejected",
 	"Name in From rejected",
+	"Name in NNTP_Host rejected",
+	"Too many Newsgroups",
 	"Not matched in Group Keep file",
 	"Multiple Matches/Tiebreaker Used"
 };
 enum { REASON_NONE, REASON_TOOMANYLINES, REASON_NOTENUFLINES, REASON_PATHHOST,
-REASON_SUBJECT, REASON_FROM, REASON_NRGRPS, REASON_NOKEEP, REASON_TIE};
+REASON_SUBJECT, REASON_FROM, REASON_NNTPHOST, REASON_NRGRPS, REASON_NOKEEP, REASON_TIE};
 enum { CHECK_EXACT, CHECK_CHECK };
 
 const struct {
 	int len;
-	char *name;
+	const char *name;
 	int headerlen;
-	char *header;
+	const char *header;
 } Params[] = {
 	{8, "HILINES=", 7, "Lines: " },
 	{9, "LOWLINES=", 7, "Lines: "},
@@ -59,65 +75,75 @@ const struct {
 	{8, "SUBJECT=", 9, "Subject: " },
 	{5, "FROM=", 6, "From: " },
 	{7, "NRGRPS=", 12, "Newsgroups: "},
+	{8, "NNTPHOST=", 19,"NNTP-Posting-Host: "},
 	{6, "GROUP=", 0, ""},
-	{6, "QUOTE=", 0, ""}
+	{6, "QUOTE=", 0, ""},
+	{8, "PROGRAM=", 0, ""}
 };
-enum { PARAM_HILINE, PARAM_LOWLINE, PARAM_PATH, PARAM_SUBJ, PARAM_FROM, PARAM_NRGRPS, PARAM_GROUP, PARAM_QUOTE};
-#define NR_PARAMS (sizeof(Params)/sizeof(Params[0]))
+
+
+enum { PARAM_HILINE, PARAM_LOWLINE, PARAM_PATH, PARAM_SUBJ, PARAM_FROM, PARAM_NRGRPS, PARAM_HOST, PARAM_GROUP, PARAM_QUOTE, PARAM_PROGRAM};
+#define NR_PARAMS ((int) (sizeof(Params)/sizeof(Params[0])))
 
 enum { DELKEEP_KEEP, DELKEEP_DELETE };
 /*--------------------------------------------------------------------------*/
 PKillStruct parse_killfile() {
-	static KillStruct mykill;
-	PKillStruct retptr;
 	FILE *fptr;
 	char buf[MAXLINLEN+1];
+	int doprg = FALSE, retval = TRUE;
+
+	static KillStruct Masterkill = { 0, NULL, chk_msg_kill };	/* initialize everything first */
 
-	/* first initialize it */
-	mykill.nrgrps = 0;
-	mykill.grps = NULL;
-	retptr = &mykill;
 	/* kill file is going to get three passes, 1st one to count how many group files to process */
 	/* so can allocate memory for all the group stuff. */
+	/* also process PROGRAM line if it exists.  If we have one we don't do anything else */
 	/* 2nd pass will be to actually process the group files */
 	/* 3rd pass will be to process the master delete stuff */
 	
-	/* FIRST PASS THRU MASTER KILLFILE - only look for group delete/keeps and count em */
+	/* FIRST PASS THRU MASTER KILLFILE - look for group delete/keeps and count em  and check for PROGRAM call*/
 	if((fptr = fopen(full_path(FP_GET, FP_DATADIR, N_KILLFILE), "r")) == NULL) {
 	/*	MyPerror(full_path(FP_GET, FP_DATADIR, N_KILLFILE));	*/
 	/* this is not really an error, so don't report it as such */
-		retptr = NULL;
+		retval = FALSE;
 	}
 	else {
 #ifdef DEBUG2
 		do_debug("Pass 1 kill file: %s\n", full_path(FP_GET, FP_DATADIR, N_KILLFILE));
 #endif
-		while(fgets(buf, MAXLINLEN, fptr) != NULL) {
+		while(fgets(buf, MAXLINLEN, fptr) != NULL && doprg == FALSE) {
 #ifdef DEBUG2
 			do_debug("Read kill file line: %s\n", buf);
 #endif
-			if(strncmp(buf, Params[PARAM_GROUP].name, Params[PARAM_GROUP].len) == 0) {
-				mykill.nrgrps++;	/* how many group files must we process */
-			}			
-		}
-		fclose(fptr);
-
-		/* SECOND PASS - call routine */
-		if(mykill.nrgrps > 0) {
-			if(pass_two(&mykill) == RETVAL_ERROR) {
-				retptr = NULL;
+			if(strncmp(buf, Params[PARAM_GROUP].name, (size_t) Params[PARAM_GROUP].len) == 0) {
+				Masterkill.totgrps++;	/* how many group files must we process */
+			}						
+			if(strncmp(buf, Params[PARAM_PROGRAM].name, (size_t) Params[PARAM_PROGRAM].len) == 0) {
+				doprg = killprg_forkit(&Masterkill, &buf[Params[PARAM_PROGRAM].len]);
 			}
 		}
-		/* THIRD PASS - process master delete stuff */
-		if(retptr != NULL && parse_a_file(N_KILLFILE, "Master", &(mykill.master)) != RETVAL_OK) {
-			retptr = NULL;
+		(void) fclose(fptr);
+
+		if(doprg == TRUE) {
+			Masterkill.totgrps = 0;
+		}
+		else {
+			/* SECOND PASS - call routine */
+			if(Masterkill.totgrps > 0) {
+				if(pass_two(&Masterkill) == RETVAL_ERROR) {
+					retval = FALSE;
+				}
+			}
+			/* THIRD PASS - process master delete stuff */
+			if(retval != FALSE && parse_a_file(N_KILLFILE, "Master", &(Masterkill.master)) != RETVAL_OK) {
+				retval = FALSE;
+			}
 		}
 	}
 
-	if(retptr == NULL) {
-		free_killfile(&mykill);	/* just in case any memory got allocated */
+	if(retval == FALSE) {
+		free_killfile(&Masterkill);	/* just in case any memory got allocated */
 	}
-	return retptr;
+	return (retval == FALSE) ? NULL :  &Masterkill;
 }
 /*-------------------------------------------------------------------------------------------*/
 int pass_two(PKillStruct killp ) {
@@ -125,14 +151,15 @@ int pass_two(PKillStruct killp ) {
 	int retval = RETVAL_OK;
 	FILE *fptr;
 	char buf[MAXLINLEN];
-	int i, grpon = 0;
+	int grpon = 0;
+	size_t i;
 	char *grpname, *grpfile, *delkeep;
 
 	grpname = grpfile = delkeep = NULL;
 
 	/* SECOND PASS - now that we know how many, we can allocate the space for em */
 	/* and then have parse_a_file read em in. */
-	if((killp->grps = calloc(killp->nrgrps, sizeof(Group))) == NULL) {
+	if((killp->grps = calloc((size_t) killp->totgrps, sizeof(Group))) == NULL) {
 		retval = RETVAL_ERROR;
 		error_log(ERRLOG_REPORT, "Out of memory, can't do killfiles\n");
 	}
@@ -148,15 +175,15 @@ int pass_two(PKillStruct killp ) {
 #ifdef DEBUG2
 			do_debug("Read kill file line: %s\n", buf);
 #endif
-			if(strncmp(buf, Params[PARAM_GROUP].name, Params[PARAM_GROUP].len) == 0 ) {
+			if(strncmp(buf, Params[PARAM_GROUP].name, (size_t) Params[PARAM_GROUP].len) == 0 ) {
 				
 				/* now parse the line for the 3 required elements */
 				/* keep/delete group_name filename */
 				delkeep = &buf[Params[PARAM_GROUP].len];
-				if(strncmp(delkeep, GROUP_KEEP, strlen(GROUP_KEEP)) == 0) {
+				if(strncmp(delkeep, GROUP_KEEP, (size_t) strlen(GROUP_KEEP)) == 0) {
 					killp->grps[grpon].delkeep = DELKEEP_KEEP;
 				}
-				else if(strncmp(delkeep, GROUP_DELETE, strlen(GROUP_DELETE)) == 0) {
+				else if(strncmp(delkeep, GROUP_DELETE, (size_t) strlen(GROUP_DELETE)) == 0) {
 					killp->grps[grpon].delkeep = DELKEEP_DELETE;
 				}
 				else {
@@ -189,6 +216,7 @@ int pass_two(PKillStruct killp ) {
 					error_log(ERRLOG_REPORT, "Invalid Parameter Line: %s\n", buf);
 				}
 				else {	/* have all three params, put them in place and parse the file */
+					/* +1 for newline */
 					if((killp->grps[grpon].group = malloc(strlen(grpname)+1)) == NULL) {
 						error_log(ERRLOG_REPORT, "Out of Memory");
 						retval = RETVAL_ERROR;
@@ -201,7 +229,7 @@ int pass_two(PKillStruct killp ) {
 				grpon++;	/* finished with this group */
 			}
 		}
-		fclose(fptr);
+		(void) fclose(fptr);
 	}
 
 	return retval;
@@ -212,21 +240,23 @@ void free_killfile(PKillStruct master) {
 	int i;
 
 	if(master != NULL) {
+		/* first kill off killprg if its there */
+		if(master->killfunc==chk_msg_kill_fork) {
+			killprg_closeit(master);
+		}
 		free_node(master->master);
-		if(master->nrgrps > 0) {
-			for(i=0;i<master->nrgrps;i++) {
+		if(master->totgrps > 0) {
+			for(i=0;i<master->totgrps;i++) {
 				free_node(master->grps[i].match);
 				free(master->grps[i].group);
 			}
 			free(master->grps);
 		}
-		if(master->master.path != NULL) {
-			free(master->master.path);
-		}
 	}
 }
 /*--------------------------------------------------------------------*/
 void free_node(OneKill node) {
+#ifndef USE_REGEX
 	if(node.path != NULL) {
 		free(node.path);
 	}
@@ -236,11 +266,43 @@ void free_node(OneKill node) {
 	if(node.subj != NULL) {
 		free(node.subj);
 	}
+	if(node.nntphost != NULL) {
+		free(node.nntphost);
+	}
+#else
+	int i;
+
+	if(node.path.ptrs != NULL) {
+		for(i = 0; i < node.path.nr; i++) {
+			regfree(&(node.path.ptrs[i]));
+		}
+		free(node.path.ptrs);
+	}
+	if(node.from.ptrs != NULL) {
+		for(i = 0; i < node.from.nr; i++) {
+			regfree(&(node.from.ptrs[i]));
+		}
+		free(node.from.ptrs);
+	}
+	if(node.subj.ptrs != NULL) {
+		for(i = 0; i < node.subj.nr; i++) {
+			regfree(&(node.subj.ptrs[i]));
+		}
+		free(node.subj.ptrs);
+	}
+	if(node.nntphost.ptrs != NULL) {
+		for(i = 0; i < node.nntphost.nr; i++) {
+			regfree((&node.nntphost.ptrs[i]));
+		}
+		free(node.nntphost.ptrs);
+	}
+#endif
 }
 /*--------------------------------------------------------------------------*/
 int get_one_article_kill(PMaster master, int logcount, PKillStruct killp) {
 
-	char buf[MAXLINLEN+1], *inbuf, *fname;
+	char buf[MAXLINLEN+1], *inbuf;
+	const char *fname;
 	int retval;
 	FILE *fptr;
 
@@ -249,7 +311,7 @@ int get_one_article_kill(PMaster master, int logcount, PKillStruct killp) {
 	/* build command to get article header*/
 	sprintf(buf, "head %s\r\n", (master->curr)->msgnr);
 
-	switch(send_a_command(master->sockfd, buf, 221)) {
+	switch(send_command(master, buf, NULL, 221)) {
 	  case RETVAL_ERROR:
 		retval = RETVAL_ERROR;
 		break;
@@ -257,7 +319,9 @@ int get_one_article_kill(PMaster master, int logcount, PKillStruct killp) {
 		if((inbuf = get_a_chunk_mem(master)) == NULL) {
 			retval = RETVAL_ERROR;
 		}
-		else if(chk_msg_kill(killp, inbuf)==FALSE){
+		/*else if(chk_msg_kill(killp, inbuf)==FALSE){*/
+		/* the killfunc pointer points to either chk_msg_kill() or chk_msg_kill_fork() */
+		  else if((*killp->killfunc)(killp, inbuf) == FALSE) {
 			if(master->MultiFile == TRUE) {
 				/* open file */
 				/* file name will be ####-#### ex 001-166 (nron,total) */
@@ -271,7 +335,7 @@ int get_one_article_kill(PMaster master, int logcount, PKillStruct killp) {
 					fputs(inbuf, fptr);
 					fputs("\n", fptr);	/* needed */
 					sprintf(buf, "body %s\r\n", (master->curr)->msgnr);
-					switch(send_a_command(master->sockfd, buf, 222)) {
+					switch(send_command(master, buf, NULL, 222)) {
 					  case RETVAL_OK:
 						retval = get_a_chunk(master->sockfd, fptr);
 						break;
@@ -281,7 +345,7 @@ int get_one_article_kill(PMaster master, int logcount, PKillStruct killp) {
 					  case RETVAL_UNEXPECTEDANS:
 						break;
 					}
-					fclose(fptr);
+					(void) fclose(fptr);
 					if(retval != RETVAL_OK) {
 						unlink(fname);
 					}
@@ -291,7 +355,7 @@ int get_one_article_kill(PMaster master, int logcount, PKillStruct killp) {
 				fputs(inbuf, stdout);
 				fputs("\n", stdout);
 				sprintf(buf, "body %s\r\n", (master->curr)->msgnr);
-				switch(send_a_command(master->sockfd, buf, 222)) {
+				switch(send_command(master, buf, NULL, 222)) {
 				  case RETVAL_OK:
 					retval = get_a_chunk(master->sockfd, stdout);
 					/* this is needed as a separator */
@@ -342,7 +406,7 @@ char *get_a_chunk_mem(PMaster master) {
 	while(buf != NULL && done == FALSE) {
 		len=sgetline(master->sockfd, &inbuf);
 #ifdef TIMER
-		TimerFunc(TIMER_ADDBYTES, len);
+		(void) TimerFunc(TIMER_ADDBYTES, len);
 #endif
 		if(len < 0) {
 			free(buf);
@@ -395,7 +459,7 @@ char *get_a_chunk_mem(PMaster master) {
 int chk_msg_kill(PKillStruct killp, char *headerbuf) {
 
 	int why, goodwhy, killyn, i, del, keep, match;
-	char *group = "Master";
+	const char *group = "Master";
 
 #ifdef KILLFILE_LOG
 	FILE *fptr;
@@ -410,7 +474,7 @@ int chk_msg_kill(PKillStruct killp, char *headerbuf) {
 		/* default actions */
  		keep = FALSE;
 		del = FALSE;
-		for(i=0;i<killp->nrgrps;i++) {
+		for(i=0;i<killp->totgrps;i++) {
 			if(check_line(headerbuf, "Newsgroups: ", killp->grps[i].group, ',', CHECK_EXACT, ' ') == TRUE) {
 				/* bingo this article matches one of our group check it */
 				match = check_a_group(&(killp->grps[i].match), headerbuf, &why);
@@ -510,54 +574,83 @@ int check_a_group(POneKill killp, char *headerbuf, int *why) {
 		
 	}
 	/* now check nrgrps */
-	if(match == FALSE && killp->nrgrps > 0) {
+	if(match == FALSE && killp->maxgrps > 0) {
 		if((startline = strstr(headerbuf, Params[PARAM_NRGRPS].header)) != NULL) {
 			/* count the nr of commas in the group line */
 			i = 1;	/* have at least one group */
 			tptr = startline;
-			while(i <= killp->nrgrps && *tptr != '\0' ) {
+			while(i <= killp->maxgrps && *tptr != '\r' && *tptr != '\0' ) {
 				if(*tptr++ == COMMA) {
 					i++;
 				}
 			}
-			if(i >= killp->nrgrps) {
+			if(i > killp->maxgrps) {
 				match = TRUE;
 				*why = REASON_NRGRPS;
 			}		
 		}
 	}
 	/* check host */
+#ifdef USE_REGEX
+	if(match == FALSE && killp->path.ptrs != NULL) {
+		if(regex_check(headerbuf, Params[PARAM_PATH].header, &(killp->path)) == TRUE) {
+#else
 	if(match == FALSE && killp->path != NULL) {
 		if(check_line(headerbuf, Params[PARAM_PATH].header, killp->path, PATHHOST_SEPARATOR, CHECK_CHECK, killp->quote) == TRUE) {
+#endif
 			/* jackpot */
 			match = TRUE;
 			*why = REASON_PATHHOST;
 		}
 	}
 	/* check from */
+#ifdef USE_REGEX
+	if(match == FALSE && killp->from.ptrs != NULL) {
+		if(regex_check(headerbuf, Params[PARAM_FROM].header, &(killp->from)) == TRUE) {
+#else
 	if(match == FALSE && killp->from != NULL) {
 		if(check_line(headerbuf, Params[PARAM_FROM].header, killp->from, FROM_SEPARATOR, CHECK_CHECK, killp->quote) == TRUE) {
+#endif
 			/* jackpot */
 			match = TRUE;
 			*why = REASON_FROM;
 		}
 	}
 	/* check subject */
+#ifdef USE_REGEX
+	if(match == FALSE && killp->subj.ptrs != NULL) {
+		if(regex_check(headerbuf, Params[PARAM_SUBJ].header, &(killp->subj)) == TRUE) {
+#else
 	if(match == FALSE && killp->subj != NULL) {
 		if(check_line(headerbuf, Params[PARAM_SUBJ].header, killp->subj, SUBJECT_SEPARATOR, CHECK_CHECK, killp->quote) == TRUE) {
+#endif
 			/* jackpot */
 			match = TRUE;
 			*why = REASON_SUBJECT;
 		}
 	}
+	/* check NNTP_Posting_Host */
+#ifdef USE_REGEX
+	if(match == FALSE && killp->nntphost.ptrs != NULL) {
+		if(regex_check(headerbuf, Params[PARAM_HOST].header, &(killp->nntphost)) == TRUE) {
+#else
+	if(match == FALSE && killp->nntphost != NULL) {
+		if(check_line(headerbuf, Params[PARAM_HOST].header, killp->nntphost, NNTP_SEPARATOR, CHECK_CHECK, killp->quote) == TRUE) {
+#endif
+			/* jackpot */
+			match = TRUE;
+			*why = REASON_NNTPHOST;
+		}
+	}
 	return match;
 }
+#ifndef USE_REGEX
 /*-----------------------------------------------------------------------*/
-char *malloc_line(int which, char *linein) {
+char *malloc_line(int which, const char *linein) {
 
 	/* malloc and copy a header line WITHOUT the header field */
 	char *ptr;
-	int x;
+	size_t x;
 
 				       /* len = how much not to malloc */
 	if((ptr = malloc((strlen(linein)+1) - Params[which].len)) == NULL) {
@@ -573,16 +666,17 @@ char *malloc_line(int which, char *linein) {
 	}
 	return ptr;
 }
+#endif /* USE_REGEX */
 /*--------------------------------------------------------------------------*/
-int check_line(char *header, char *whichline, char *linematch, int separator, int check_quote, char quote_char) {
+int check_line(char *header, const char *whichline, const char *linematch, int separator, int check_quote, char quote_char) {
 	/* search for each item in linematch on whichline in the header */
 	/* if check_quote = CHECK_CHECK check if first char is char_quote */
 	/* if it is, then do strcmp, else do a strncmp */
 	/* if check_quote = CHECK_EXACT don't check first char just strcmp */
 
-	char *startline, *endline, *currcomma, *nextcomma;
+	char *startline, *endline, *nextcomma;
+	const char *currcomma;
 	int casecmp, match = FALSE;
-
 	
 	if((startline = strstr(header, whichline)) != NULL) {
 		currcomma = linematch;	/* start us off */
@@ -629,7 +723,7 @@ int check_line(char *header, char *whichline, char *linematch, int separator, in
 	return match;
 }
 /*-------------------------------------------------------------------------------*/
-int parse_a_file(char *fname, char *group, POneKill mykill) {
+int parse_a_file(const char *fname, const char *group, POneKill mykill) {
 
 	FILE *fptr;
 	char buf[MAXLINLEN+1];
@@ -637,8 +731,13 @@ int parse_a_file(char *fname, char *group, POneKill mykill) {
 	int retval = RETVAL_OK;
 
 	/* first initialized the killstruct */
-	mykill->hilines = mykill->lowlines = mykill->nrgrps = 0;
+	mykill->hilines = mykill->lowlines = mykill->maxgrps = 0;
+#ifndef USE_REGEX
 	mykill->path = mykill->from = mykill->subj = NULL;
+#else
+	mykill->path.ptrs = mykill->from.ptrs = mykill->subj.ptrs = mykill->nntphost.ptrs = NULL;
+#endif
+
 	mykill->quote = KILLFILE_QUOTE;
 
 	/* now read in the killfile and parse it */
@@ -655,30 +754,33 @@ int parse_a_file(char *fname, char *group, POneKill mykill) {
 #ifdef DEBUG2
 			do_debug("Read kill file line: %s\n", buf);
 #endif
-			buf[MAXLINLEN] = 0;	/* just in case */
+			buf[MAXLINLEN] = '\0';	/* just in case */
 
 			for(i = 0 ; i < NR_PARAMS; i++) {
-				if(strncmp(buf, Params[i].name, Params[i].len) == 0) {
+				if(strncmp(buf, Params[i].name, (size_t) Params[i].len) == 0) {
 					switch(i) {
 					  case PARAM_HILINE:
-						sscanf(&buf[Params[PARAM_HILINE].len], "%d", &(mykill->hilines));
+						(void) sscanf(&buf[Params[PARAM_HILINE].len], "%d", &(mykill->hilines));
 #ifdef DEBUG2
 						do_debug("Killfile hilines = %d\n", mykill->hilines);
 #endif
 						break;
 					  case PARAM_LOWLINE:
-						sscanf(&buf[Params[PARAM_LOWLINE].len], "%d", &(mykill->lowlines));
+						(void) sscanf(&buf[Params[PARAM_LOWLINE].len], "%d", &(mykill->lowlines));
 #ifdef DEBUG2
 						do_debug("Killfile lowlines = %d\n", mykill->lowlines);
 #endif
 						break;
 					  case PARAM_NRGRPS:
-						sscanf(&buf[Params[PARAM_NRGRPS].len], "%d", &(mykill->nrgrps));
+						(void) sscanf(&buf[Params[PARAM_NRGRPS].len], "%d", &(mykill->maxgrps));
 #ifdef DEBUG2
-						do_debug("Killfile nrgrps = %d\n", mykill->nrgrps);
+						do_debug("Killfile maxgrps = %d\n", mykill->maxgrps);
 #endif
 						break;						
 					  case PARAM_PATH:
+#ifdef USE_REGEX
+						regex_scan(&(mykill->path), buf, PATH_SEPARATOR, mykill->quote);
+#else
 						if((mykill->path = malloc_line(PARAM_PATH, buf)) == NULL) {
 							error_log(ERRLOG_REPORT, "No paths will be processed for %s\n", group);
 						}
@@ -686,9 +788,14 @@ int parse_a_file(char *fname, char *group, POneKill mykill) {
 						else {
 							do_debug("Killfile path = %s\n", mykill->path);
 						}
-#endif
+#endif /* DEBUG2 */
+#endif /* USE_REGEX */
 						break;
 					  case PARAM_FROM:
+#ifdef USE_REGEX
+						regex_scan(&(mykill->from), buf, FROM_SEPARATOR, mykill->quote);
+#else
+
 						if((mykill->from = malloc_line(PARAM_FROM, buf)) == NULL) {
 							error_log(ERRLOG_REPORT, "No From lines will be processed for %s\n", group);
 						}
@@ -696,9 +803,13 @@ int parse_a_file(char *fname, char *group, POneKill mykill) {
 						else {
 							do_debug("Killfile from = %s\n", mykill->from);
 						}
-#endif
+#endif /* DEBUG2 */
+#endif /* USE_REGEX */
 						break;
 					  case PARAM_SUBJ:
+#ifdef USE_REGEX
+						regex_scan(&(mykill->subj), buf, SUBJECT_SEPARATOR, mykill->quote);
+#else
 						if((mykill->subj = malloc_line(PARAM_SUBJ, buf)) == NULL) {
 							error_log(ERRLOG_REPORT, "No Subjects will be processed for %s\n", group);
 						}
@@ -706,10 +817,26 @@ int parse_a_file(char *fname, char *group, POneKill mykill) {
 						else {
 							do_debug("Killfile Subject = %s\n", mykill->subj);
 						}
-#endif
+#endif /* DEBUG2 */
+#endif /* USE_REGEX */
 						break;
+					  case PARAM_HOST:
+#ifdef USE_REGEX
+						regex_scan(&(mykill->nntphost), buf, NNTP_SEPARATOR, mykill->quote);
+#else
+						if((mykill->nntphost = malloc_line(PARAM_HOST, buf)) == NULL) {
+							error_log(ERRLOG_REPORT, "No NNTP Hosts will be processed for %s\n", group);
+						}
+#ifdef DEBUG2 
+						else {
+							do_debug("Killfile NNTP Host = %s\n", mykill->nntphost);
+						}
+#endif /* DEBUG2 */
+#endif /* USE_REGEX */
+						break;
+
 					  case PARAM_QUOTE:
-						if(buf[Params[PARAM_QUOTE].len] == '\0' && buf[Params[PARAM_QUOTE].len] == '\n') {
+						if(buf[Params[PARAM_QUOTE].len] == '\0' || buf[Params[PARAM_QUOTE].len] == '\n') {
 							error_log(ERRLOG_REPORT, "No character to process for quote, ignoring\n");
 						}
 						else {
@@ -719,24 +846,24 @@ int parse_a_file(char *fname, char *group, POneKill mykill) {
 #endif
 						}
 						break;
-					  default:	/* ignore group lines, those processed elsewhere */
+					  default:	/* ignore group  and program lines, those processed elsewhere */
 						break;
 					  
 					}
 				}
 			}
 		}
-		fclose(fptr);
+		(void) fclose(fptr);
 	}
 	return retval;
 }
 /*---------------------------------------------------------------------------------------------*/
-char *strnstr(char *linein, char *matchstr) {
+const char *strnstr(const char *linein, const char *matchstr) {
 
 	/* see if matchstr exists in inline, case insensitive */
 	/* return start of string in inline, else return NULL */
 
-	char *tptr, *startchar, *retstr = NULL;
+	const char *tptr, *startchar, *retstr = NULL;
 	
 	if(linein != NULL && matchstr != NULL) {
 		while(*linein != '\0' && retstr == NULL) {
@@ -766,3 +893,93 @@ char *strnstr(char *linein, char *matchstr) {
 	}
 	return retstr;
 }
+#ifdef USE_REGEX
+/*----------------------------------------------------------------------------------*/
+/* scan a line and build an array of compiled regex pointers to call regexex() with */
+/* ---------------------------------------------------------------------------------*/
+void regex_scan(pmy_regex which, char *linein, char separator, char quotechar) {
+
+	regex_t *retval = NULL;
+	char *tptr, *startline, errmsg[256];
+	int err, i, flags, nr = 0;
+
+	if(linein != NULL) {
+#ifdef DEBUG2
+		do_debug("String to parse =%s", linein);
+#endif
+		/* first find the = in the line to skip the parameter part */
+		tptr = linein;
+		while(*tptr != '\0' && *tptr != '=') {
+			tptr++;
+		}
+		startline = tptr+1;
+		/* now count each of em, so that can alloc my array of regex_t */
+		if(*tptr != '\0') {
+			nr = 1;
+			while(*tptr != '\0') {
+				if(*tptr++ == separator) {
+					nr++;
+				}
+			}
+			/* now alloc memory */
+			if((retval = calloc(sizeof(regex_t), nr)) == NULL) {
+				error_log(ERRLOG_REPORT, "Unable to alloc memory to process killfile regex, skipping\n");
+			}
+			else {
+				/* now for each string, do the regcomp */
+				for(i=0;i<nr;i++) {
+					tptr = startline;
+					while(*tptr != '\0' && *tptr != '\n' && *tptr != separator) {
+						tptr++;
+					}
+					/* at end of separator */
+					*tptr++ = '\0';	/* so can call regcomp() correctly */
+							/* the ++ so next one set up correctly */
+					if(*startline == quotechar) {
+						tptr++;
+						flags = REG_NOSUB;
+					}
+					else {
+						flags = REG_NOSUB | REG_ICASE;
+					}
+#ifdef DEBUG2
+					do_debug("regcomping=%s, flags=%d\n", startline, flags);
+#endif
+					if((err = regcomp(&(retval[i]), startline, flags)) != 0) {
+						/* whoops */
+						regerror(err, &(retval[i]), errmsg, sizeof(errmsg));
+						error_log(ERRLOG_REPORT, "%s\n", errmsg);
+					}
+					startline=tptr;		/* so at start of next one */
+				}
+			}
+		}			
+	}
+	which->ptrs = retval;
+	which->nr = nr;
+}
+/*------------------------------------------------------------------------------------*/
+int regex_check(char *headerbuf, const char *whichline, pmy_regex expr) {
+	int i, match = FALSE;
+	char *startline, *endline;
+
+	if((startline = strstr(headerbuf, whichline)) != NULL) {
+		endline = strchr(startline, '\n');	
+		/* end this line, so we only search the right header line */
+		if(endline != NULL) {
+			*endline = '\0';
+		}
+		for(i = 0; i < expr->nr && match == FALSE; i++) {
+			if(regexec(&(expr->ptrs[i]), startline, 0, NULL, 0) == 0) {
+				match = TRUE;
+			}
+		}
+		/* put the nl back on that we deleted */
+		if(endline != NULL) {
+			*endline = '\n';
+		}	
+	}
+	return match;
+}
+#endif
+
