@@ -1,13 +1,16 @@
+#include <config.h>
+
 #include <stdio.h>
 #include <errno.h>	
-#include "config.h"
+#include "suck_config.h"
 
 /* get news data from stdin and post it to the local server */
 
-void main(int argc,char *argv[]) {
+int main(int argc,char *argv[]) {
 	FILE *pfp=NULL;
 	char line[1024];
-	int len,count=0,verbose=0, retval=0;
+	int count=0,verbose=0, retval=0;
+	size_t len;
 
 	if (argc>1)  {
 		verbose=1;
@@ -16,7 +19,7 @@ void main(int argc,char *argv[]) {
 	while(gets(line) != NULL && retval == 0) {
   		len=strlen(line);
 		if (pfp == NULL) {
-			if (verbose) {
+			if (verbose != 0) {
 				printf("posting article %d\n", ++count);
 			}
 			pfp = popen(RNEWS, "w");
@@ -27,7 +30,7 @@ void main(int argc,char *argv[]) {
 		}
 		else if (line[0] == '.' && len == 1) {
 			/* end of article */
-			if (verbose) {
+			if (verbose != 0) {
 				printf("end of article %d\n",count);
 			}
 			if(pfp != NULL) {
@@ -36,8 +39,8 @@ void main(int argc,char *argv[]) {
 			}
 		}
 		else {
-			fputs(line, pfp);
-			putc('\n', pfp);
+			(void) fputs(line, pfp);
+			(void) putc('\n', pfp);
 		}
 	} /* end while */
 	exit(retval);
