@@ -1,15 +1,39 @@
+#include <config.h>
+
 #include <stdio.h>
 #include <stdlib.h>
+
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
+
+#ifdef HAVE_DIRENT_H
+# include <dirent.h>
+#else
+# define dirent direct
+# ifdef HAVE_SYS_NDIR_H
+#  include <sys/ndir.h>
+# endif
+# ifdef HAVE_SYS_DIR_H
+#  include <sys/dir.h>
+# endif
+#endif
+
+#ifdef HAVE_LIMITS_H
+#include <limits.h>
+#endif
+
 #include <netdb.h>
-#include <dirent.h>
 #include <string.h>
 #include <sys/stat.h>
 #include <math.h>
-#include "config.h"
+#include <errno.h>
+
+#include "suck_config.h"
 #include "both.h"
 #include "suck.h"
 #include "suckutils.h"
+#include "dedupe.h"
 
 #ifdef KILLFILE
 #include "killfile.h"
@@ -33,14 +57,14 @@ int get_articles(PMaster);
 int get_one_article(PMaster, int);
 int get_announcements(PMaster);
 int do_supplemental(PMaster);
-int do_innbatch(int, char *);
-int do_rnewsbatch(int, char *, long);
-PList allocnode(char *, int);
-int build_list(PMaster, char *, int *, int);
+int do_innbatch(PMaster);
+int do_rnewsbatch(PMaster);
+int allocnode(PMaster, char *, int);
 int restart_yn(PMaster);
-
+int scan_args(PMaster, int, char *[]);
+void do_cleanup(void);
 #ifdef MYSIGNAL
-void sighandler(int);
+RETSIGTYPE sighandler(int);
 
 /*------------------------------------------*/
 int GotSignal = FALSE;		
@@ -59,166 +83,95 @@ void main(int argc, char *argv[]) {
 	struct hostent *hi;
 	Master master;
 	PList temp;
-	char *lockfile;
+	char **args, **fargs = NULL;
+	int loop, fargc, retval = RETVAL_OK;
 
-	char *host = NULL, *batchfile = NULL, *status_file = NULL, *ptr;
-	int loop, batch = BATCH_FALSE, retval = RETVAL_OK, always_batch = FALSE;
-	long rnews_size = 0L;
+#ifdef LOCKFILE
+	const char *lockfile = NULL;
+#endif
 
+	/* initialize master structure */
 	master.head = master.curr = NULL;
 	master.nritems =  master.itemon = master.nrgot = 0;
 	master.MultiFile = FALSE;
 	master.msgs = stdout;
 	master.sockfd = -1;
 	master.status_file = FALSE;	/* are status messages going to a file */
+	master.status_file_name = NULL;
 	master.do_killfile = TRUE;
 	master.do_chkhistory = TRUE;
+	master.do_modereader = FALSE;
+	master.always_batch = FALSE;
+	master.rnews_size = 0L;
+	master.batch = BATCH_FALSE;
+	master.batchfile = NULL;
+	master.cleanup = FALSE;
+	master.portnr = DEFAULT_NNRP_PORT;
+	master.host = getenv("NNTPSERVER");		/* the default */
+	master.pause_time = -1;
+	master.pause_nrmsgs = -1;
+	master.ms_kludge = FALSE;	/* microsoft kludge for xhdr command */
 
 	/* allow no args, only the hostname, or hostname as first arg */
-	host = getenv("NNTPSERVER");		/* the default */
+	/* also have to do the file argument checking */
 	switch(argc) {
 	  case 1:
 		break;
-	  case 2:   
-		host = argv[1];
+	  case 2:
+		/* the fargs == NULL test so only process first file name */
+		if(argv[1][0] == FILE_CHAR) {
+			if((fargs = build_args(&argv[1][1], &fargc)) != NULL) {
+				retval = scan_args(&master, fargc, fargs);
+			}
+		}   
+		else {
+			master.host = argv[1];
+		}
 		break;
 	  default:
-		loop = 1;
-		if(argv[1][0] != '-') {
-			host = 	argv[loop++];
-		}			
-		for(;loop<argc;loop++) {
-			if(argv[loop][0] == '-') {
-				switch(argv[loop][1]) {
-				  case 'a':	/* if we have downloaded at least one article, then batch up */
-						/* even on errors */
-					always_batch = TRUE;
-					break;
-				  case 'h':
-					host = (loop+1 == argc) ? getenv("NNTPSERVER") : argv[++loop];
-					break;	
-				  case 'm':
-					master.MultiFile = TRUE;
-					break;
-				  case 'r':
-					if(loop+1 == argc) {
-						error_log(ERRLOG_REPORT, "No rnews batch file size provided\n");
-						retval = RETVAL_ERROR;
-					}
-					else {
-						rnews_size = atol(argv[++loop]);
-					}
-					break;
-				  case 'p':
-					if(loop+1 == argc) {
-						error_log(ERRLOG_REPORT, "No postfix provided\n");
-						retval = RETVAL_ERROR;
-					}
-					else {
-						full_path(FP_SET_POSTFIX, FP_NONE, argv[++loop]);
-					}
-					break;
-				  case 'd':	/* change one of the directory names */
-					if(loop+1 == argc) { 
-						error_log(ERRLOG_REPORT, "No directory name provided\n");
-						retval = RETVAL_ERROR;
-					}
-					else {
-						ptr = NULL;
-						switch (argv[loop][2]) {
-						  case 't':
-							ptr = full_path(FP_SET, FP_TMPDIR, argv[++loop]);
-							break;
-						  case 'd':
-							ptr = full_path(FP_SET, FP_DATADIR, argv[++loop]);
-							break;
-						  case 'm':
-							ptr = full_path(FP_SET, FP_MSGDIR, argv[++loop]);
-							break;
-						}
-						if(ptr == NULL) { 
-							error_log(ERRLOG_REPORT, "Invalid directory\n");
-							retval = RETVAL_ERROR;
-							break;
-						}
-					}
-					break;		 
-				  case 'b':	/* batch file implies MultiFile Mode */
-					switch (argv[loop][2]) {
-					  case 'i':
-						batch = BATCH_INNXMIT;
-						master.MultiFile = TRUE;		
-						break;
-					  case 'r':
-						batch = BATCH_RNEWS;
-						master.MultiFile = TRUE;
-						break;
-					  default:
-						error_log(ERRLOG_REPORT, "Invalid Batch arg: %s\n", argv[loop]);
-						retval = RETVAL_ERROR;
-						break;
-				  	}
-					if(retval == RETVAL_OK && loop+1 == argc) {
-						error_log(ERRLOG_REPORT, "No Batch file name provided\n");
-						retval = RETVAL_ERROR;
-					}
-					else {
-						batchfile = argv[++loop];
-					}
-					break;
-				  case 'e':	/* use default error log path */
-					error_log(ERRLOG_SET_FILE, ERROR_LOG);
-					break;
-				  case 'E':	/* error log path */
-					if(loop+1 == argc) { 
-						error_log(ERRLOG_REPORT, "No Error Log name provided\n");
-						retval = RETVAL_ERROR;
-					}
-					else {
-						error_log(ERRLOG_SET_FILE, argv[++loop]);
-					}
-					break;
-				  case 's':	/* use default status log name */
-					status_file = STATUS_LOG;
-					break;
-				  case 'S':	/* status log path */
-					if(loop+1 == argc) {
-						error_log(ERRLOG_REPORT, "No Status Log name provided\n");
-						retval = RETVAL_ERROR;
-					}
-					else {
-						status_file = argv[++loop];
-					}
-					break;
-				  case 'V': 	/* show version number */
-				  	error_log(ERRLOG_REPORT, "Suck version %d.%d.%d\n",SUCKVER, SUCKVER_MINOR, SUCKVER_PATCH);
-					retval = RETVAL_VERNR;	/* so we don't do anything else */
-				  	break;
-				  case 'K':
-					master.do_killfile = FALSE;
-					break;
-				  case 'H':
-					master.do_chkhistory = FALSE;
-					break;
-				  default:
-					error_log(ERRLOG_REPORT, "Invalid argument: %s\n", argv[loop]);
-					retval = RETVAL_ERROR;
-					break;
-				}			
-			}
-			else {
-				error_log(ERRLOG_REPORT, "Invalid argument: %s\n", argv[loop]);
-				retval = RETVAL_ERROR;
-			}	
+		for(loop=1;loop<argc && fargs == NULL;loop++) {
+			if(argv[loop][0] == FILE_CHAR) {
+				if((fargs = build_args(&argv[loop][1], &fargc)) != NULL) {
+					retval = scan_args(&master, fargc, fargs);
+				}
+			}   
 		}
+		/* this is here so anything at command line overrides file */
+		if(argv[1][0] != '-' && argv[1][0] != FILE_CHAR) {
+			master.host = 	argv[1];
+			argc-= 2;
+			args = &(argv[2]);
+		}
+		else {
+			args = &(argv[1]);
+			argc--;
+		}
+		retval = scan_args(&master, argc, args);	
 		break;
 	}
 	if(retval == RETVAL_OK) {
-		
+	
+#ifdef DEBUG2
+		do_debug("Suck version %d.%d.%d\n",SUCKVER, SUCKVER_MINOR, SUCKVER_PATCH);
+		do_debug("master.MultiFile = %d\n", master.MultiFile);
+		do_debug("master.msgs = %d\n", master.msgs);
+		do_debug("master.status_file = %d\n",master.status_file);
+		do_debug("master.status_file_name = %s\n", (master.status_file_name == NULL) ? "NULL" : master.status_file_name);
+		do_debug("master.do_killfile = %d\n", master.do_killfile);
+		do_debug("master.do_chkhistory = %d\n", master.do_chkhistory);
+		do_debug("master.do_modereader = %d\n", master.do_modereader);
+		do_debug("master.always_batch = %d\n", master.always_batch);
+		do_debug("master.rnews_size = %ld\n", master.rnews_size);
+		do_debug("master.batch = %d\n", master.batch);
+		do_debug("master.batchfile = %s\n", (master.batchfile == NULL) ? "NULL" : master.batchfile );
+		do_debug("master.cleanup = %d\n", master.cleanup);
+		do_debug("master.host = %s\n", (master.host == NULL) ? "NULL" : master.host);
+#endif
+	
 #ifdef LOCKFILE
 		/* this has to be here since we use full_path() to get path for lock file */
 		/* and it isn't set up until we process the args above. */
-		if(do_lockfile() != RETVAL_OK) {
+		if(do_lockfile(&master) != RETVAL_OK) {
 			exit(-1);
 		}
 #endif
@@ -231,9 +184,9 @@ void main(int argc, char *argv[]) {
 		/* set up status log, if none specified or unable to open status log */
 		/* then  use stdout or stderr */
 
-		if(status_file != NULL) {
+		if(master.status_file_name != NULL) {
 			/* okay attempt to open it up */
-			if((master.msgs = fopen(status_file, "a")) == NULL) {
+			if((master.msgs = fopen(master.status_file_name, "a")) == NULL) {
 				MyPerror("suck: status log");
 			}
 			else {
@@ -244,12 +197,13 @@ void main(int argc, char *argv[]) {
 			/* if not in multifile mode, all status msgs MUST go to stderr to not mess up articles */
 			master.msgs = ( master.MultiFile == FALSE) ? stderr : stdout ;
 		}
-					
-		master.sockfd = connect_to_nntphost( host, &hi, master.msgs);
+		
+		fprintf(master.msgs, "Attempting to connect to %s\n", master.host);		
+		master.sockfd = connect_to_nntphost( master.host, &hi, master.msgs, master.portnr);
 		if(master.sockfd < 0 ) {
 			retval = RETVAL_ERROR;
 		}
-		else if(get_announcements(&master) < RETVAL_OK) {
+		else if(get_announcements(&master) != RETVAL_OK) {
 			retval = RETVAL_ERROR;
 		}
 		else {
@@ -270,26 +224,30 @@ void main(int argc, char *argv[]) {
 			else {
 				retval = get_articles(&master);
 			}	
-			sputline( master.sockfd, "quit\r\n" );
+			send_command( &master, "quit\r\n", NULL, 0 );
 		}
 		if(master.sockfd >= 0) {
 			close(master.sockfd);
 			fprintf(master.msgs,"Closed connection to %s\n",hi->h_name);
 		}
-		if((retval == RETVAL_OK || always_batch == TRUE) && master.nrgot > 0) {
-			switch(batch) {
+		if((retval == RETVAL_OK || master.always_batch == TRUE) && master.nrgot > 0) {
+			switch(master.batch) {
 			  case BATCH_INNXMIT:
 				fprintf(master.msgs,"Building Inn Batch File\n");
-				do_innbatch(master.nritems, batchfile);
+				do_innbatch(&master);
 				break;
 			  case BATCH_RNEWS:
 				fprintf(master.msgs,"Building Rnews Batch File(s)\n");
-				do_rnewsbatch(master.nritems, batchfile, rnews_size);
+				do_rnewsbatch(&master);
 				break;
 			  default:
 				break;
 			}
 		}
+		if((retval == RETVAL_NOARTICLES || retval == RETVAL_OK) && master.cleanup == TRUE) {
+			fprintf(master.msgs, "Cleaning up after myself\n");
+			do_cleanup();
+		}
 		/* close out status log */
 		if(master.msgs != NULL && master.msgs != stdout && master.msgs != stderr) {
 			fclose(master.msgs);
@@ -302,6 +260,9 @@ void main(int argc, char *argv[]) {
 				free_one_node(master.curr);
 				master.curr = temp;
 			}
+		}
+		if(fargs != NULL) {
+			free_args(fargc, fargs);
 		}		
 #ifdef LOCKFILE
 		lockfile = full_path(FP_GET, FP_TMPDIR, N_LOCKFILE);	
@@ -315,13 +276,21 @@ void main(int argc, char *argv[]) {
 /*--------------------------------------------------------------------*/
 int get_message_index(PMaster master) {
 
-	int count,low,high,response,lastread,retval,i,removed;
+	int count,low,high,response,lastread,retval,i,maxread,xhdr_result;
 	char *sp,*inbuf,buf[MAXLINLEN],group[512];
 	FILE *ifp,*tmpfp;
 
 	retval = RETVAL_OK;
 	ifp = tmpfp = NULL;
-	removed = 0;
+
+#ifdef TIMER
+	TimerFunc(TIMER_START, 0);
+#endif
+	/* msnews.microsoft.com doesn't use the standard return value */
+	/* for the xhdr command, it uses 224, so I have to be able to */
+	/* handle either one */
+	xhdr_result = ( master->ms_kludge == FALSE) ? 221 : 224 ;
+
 	if((ifp = fopen(full_path(FP_GET, FP_DATADIR, N_OLDRC), "r" )) == NULL) {
 		MyPerror(full_path(FP_GET, FP_DATADIR, N_OLDRC));
 		retval = RETVAL_ERROR;
@@ -337,24 +306,25 @@ int get_message_index(PMaster master) {
 			fprintf(master->msgs, "Skipping Line: %s", buf);
 			continue;
 		}
-		if(sscanf(buf, "%s %d\n", group, &lastread) != 2) {
+		i = sscanf(buf, "%s %d %d\n", group, &lastread, &maxread);
+		if ( i < 2 || i > 3) {
 			error_log(ERRLOG_REPORT, "Invalid Line: %s", buf);
 			continue;
 		}
-  		sprintf(buf,"group %s\r\n",group);
-		sputline(master->sockfd,buf);
-		i = sgetline(master->sockfd,&inbuf);
-#ifdef DEBUG2
-		do_debug("\ngroup %s\n%s",group,inbuf);
-#endif
-		if(i < 0) {
-			retval = RETVAL_ERROR;
+		/* now make some sanity tests on the maxread value */
+		if(i == 2) { 
+			maxread = 0;
 		}
-		else if((sp = number(inbuf, &response)) == NULL) {
-			error_log(ERRLOG_REPORT,"Protocol error!\r\n");
+		else if (maxread <= 0) {
+			error_log(ERRLOG_REPORT, "Invalid number for maxread field: %d : ignoring\n", maxread);
+			maxread = 0;
+		}
+  		sprintf(buf,"group %s\r\n",group);
+		if(send_command(master,buf,&inbuf,0) != RETVAL_OK) {
 			retval = RETVAL_ERROR;
 		}
 		else {
+			sp = number(inbuf, &response);
 			if(response != 211) {
 				/* handle group not found - rewrite line in newrc */
 				if(response == 411) {
@@ -375,51 +345,64 @@ int get_message_index(PMaster master) {
 			/* the > 0 is needed, since if a nnrp site has no article it will reset */
 			/* the count to 0.  So not an error */
   			if(lastread > high && high > 0) {
-  				fprintf(master->msgs,"%s...High Article Nr is low, did host reset its counter?",group);
+  				fprintf(master->msgs,"%s - %d...High Article Nr is low, did host reset its counter?\n",group,high);
   			}
 
+			if(lastread < 0 ) {
+				/* if a neg number, get the last X nr of messages, handy for starting */
+				/* a new group off ,say -100 will get the last 100 messages */
+				lastread += high;	/* this works since lastread is neg */
+				if(lastread < 0) {
+					lastread = 0;	/* just to be on the safeside */
+				}
+			}
+
 			/* this has to be >= 0 since if there are no article on server high = 0 */
 			/* so if we write out 0, we must be able to handle zero as valid lastread */
-			if (lastread < high && lastread >= 0) {
+			/* the count > 0 so if no messages available we don't even try */
+			if (count > 0 && lastread < high && lastread >= 0) {
    				if(lastread < low) {
 					lastread = low - 1;
 				}
+				/* now set the max nr of messages to read */
+				if(maxread > 0 && high-maxread > lastread) {
+					lastread = high-maxread;
+					fprintf(master->msgs, "%s - limiting # of articles to %d\n",group, maxread);
+				}
    
    				sprintf(buf, "xhdr Message-ID %d-\r\n", lastread+1);
    				fprintf(master->msgs,"%s...articles %d-%d\r\n", group, lastread+1, high);
-   				sputline(master->sockfd, buf);
-				i = sgetline(master->sockfd, &inbuf);
-				number(inbuf, &response);
-				if(i < 0) {
+				i = send_command(master,buf,&inbuf,xhdr_result);
+				if(i != RETVAL_OK) {
 					retval = RETVAL_ERROR;
 				}
-				else if (response == 221) {
+				else {
 					do {
      						if(sgetline(master->sockfd, &inbuf) < 0) {
 							retval = RETVAL_ERROR;
 						}
 						else if (*inbuf != '.' ) {
-      							for(sp = inbuf; *sp != '\0' && *sp != '<'; sp++) {
-								 /* find begin of index */
-								if (sp== inbuf+(MAXLINLEN-4)) {
-									continue; /* the NNTP server tried to pull your leg */
-								}
-							}
-							retval = build_list(master, inbuf, &removed, MANDATORY_OPTIONAL);
+							retval = allocnode(master, inbuf, MANDATORY_OPTIONAL);
 						}
-					} while (retval == RETVAL_OK && strcmp(inbuf, ".\n")!=0);
+					} while (retval == RETVAL_OK && *inbuf != '.' && *(inbuf+1) != '\n');
 				} /* end if response */
-   				else {
-					error_log(ERRLOG_REPORT,"Unexpected response %d getting IDs\r\n", response);
-					retval = RETVAL_ERROR;
-				}
-			} /* end if lastread */
+  			} /* end if lastread */
 		} /* end else */
 	} /* end while */
+#ifdef TIMER
+	fprintf(master->msgs, "%s\n", TimerFunc(TIMER_TIMEONLY, 0));
+#endif
 	if(retval == RETVAL_OK) {
-		fprintf(master->msgs, "%d Articles to download, %d dupes removed\n", master->nritems, removed);
+		fprintf(master->msgs, "%d Articles to download\n", master->nritems);
 		retval = do_supplemental(master);
 	}
+	else {
+		/* this is in case we had to abort the above while loop (due to loss of pipe to server) and */
+		/* we hadn't finished writing out the suck.newrc, this finishes it up. */
+		while(fgets(buf, MAXLINLEN-1, ifp) != NULL) {
+			fputs(buf, tmpfp);
+		}
+	}
 	if(tmpfp != NULL) {
 		fclose(tmpfp);
 	}
@@ -432,14 +415,14 @@ int get_message_index(PMaster master) {
 int get_articles(PMaster master) {
 
 	int retval, logcount, i;
-	FILE *logfp,*ifp;
+	FILE *logfp;
 
 #ifdef KILLFILE
 	PKillStruct killp = NULL;
 #endif
 
 	retval = RETVAL_OK;
-	logfp = ifp = NULL;
+	logfp = NULL;
 
 	logcount = log10((double)master->nritems) + 1;	/* how many digits wide is the articleCount */
 
@@ -482,6 +465,12 @@ int get_articles(PMaster master) {
 				fputc('1',logfp);
   				fflush(logfp);
 				master->itemon++;
+				/* to be polite to the server, lets allow for a pause every once in a while */
+				if(master->pause_time > 0 && master->pause_nrmsgs > 0) {
+					if(master->itemon % master->pause_nrmsgs == 0) {
+						sleep(master->pause_time);
+					}
+				}
 				if(master->status_file == FALSE) {
 					/* if we are going to a file, we don't want all of these articles printed */
 #ifndef TIMER
@@ -511,7 +500,6 @@ int get_articles(PMaster master) {
 			}
 #endif				
 		}
-		fclose(ifp);
 	}
 #ifdef KILLFILE
 	free_killfile(killp);
@@ -528,17 +516,17 @@ int get_announcements(PMaster master) {
 		retval = RETVAL_ERROR;
 	}
 	else {
-		fprintf(master->msgs,"%s", inbuf );
-#ifndef NNRP
-		sputline(master->sockfd,"mode reader\r\n");	
-		if( sgetline(master->sockfd,&inbuf) < 0) {	
-			retval = RETVAL_ERROR;		  	
-		}					
-		else {					
-			/* Again the announcement */
-			fprintf(master->msgs,"%s",inbuf);
-		}
+#ifdef DEBUG2
+		do_debug("Got: %s", inbuf);
 #endif
+		fprintf(master->msgs,"%s", inbuf );
+		if(master->do_modereader == TRUE) {
+			retval = send_command(master, "mode reader\r\n", &inbuf, 0);
+			if(retval == RETVAL_OK) {			
+				/* Again the announcement */
+				fprintf(master->msgs,"%s",inbuf);
+			}
+		}
 	}
 	return retval;
 }
@@ -547,13 +535,12 @@ int get_announcements(PMaster master) {
 /*----------------------------------------------------------------------*/ 
 int do_supplemental(PMaster master) {
 
-	int retval, removed, oldkept;
+	int retval, oldkept;
 	FILE *fp;
 	PList curr; 
 	char linein[MAXLINLEN+1];
 
 	retval = RETVAL_OK;
-	removed = 0;
 	oldkept = master->nritems;		
 
 	if((fp = fopen(full_path(FP_GET, FP_DATADIR, N_SUPPLEMENTAL), "r")) != NULL) {
@@ -563,13 +550,18 @@ int do_supplemental(PMaster master) {
 				error_log(ERRLOG_REPORT, "Supplemental Invalid line: %s, ignoring\n", linein);
 			}
 			else {
-				retval = build_list(master, linein, &removed, MANDATORY_YES);
+				retval = allocnode(master, linein, MANDATORY_YES);
 			}
 		}
-		fprintf(master->msgs,"Supplemental List Processed - %d articles added, %d articles removed\n", 
-				master->nritems-oldkept, removed);
+		fprintf(master->msgs,"Supplemental List Processed - %d articles added, %d articles\n", 
+			master->nritems-oldkept, master->nritems);
 		fclose(fp);
 	}
+	dedupe_list(master);
+#ifdef TIMER
+	TimerFunc(TIMER_TIMEONLY, 0);
+#endif
+
 #ifdef CHECK_HISTORY
 	if(master->do_chkhistory == TRUE) {
 		chkhistory(master);
@@ -597,90 +589,59 @@ int do_supplemental(PMaster master) {
 	}
 	return retval;
 }
-/*--------------------------------------------------------------------------*/
-int build_list(PMaster master, char *linein,  int *removed, int mandatory) {
+/*-----------------------------------------------------------------------*/
+int allocnode(PMaster master, char *linein, int mandatory) {
+	/* if allocate memory here, must free in free_one_node */
+	
+	PList ptr = NULL;
+	char *end_ptr, *st_ptr;
+	static PList curr = NULL;	/* keep track of current end of list */
+	int retval = RETVAL_OK;
 
-	PList curr, prev;
-	int match, retval;
-	char *st_ptr, *ptr;
-
-	retval = RETVAL_OK;
+	/* first, find the message number */
 	st_ptr = linein;
 	while(*st_ptr != '<' && *st_ptr != '\0') {
 		st_ptr++;
 	}
-	ptr = st_ptr;
-	while(*ptr != '>' && *ptr != '\0') {
-		ptr++;
+	end_ptr = st_ptr;
+	while(*end_ptr != '>' && *end_ptr != '\0') {
+		end_ptr++;
 	}
 	
-	if(*st_ptr != '<' && *ptr != '>') {
+	if(*st_ptr != '<' && *end_ptr != '>') {
 		error_log(ERRLOG_REPORT, "Invalid Article NR, skipping: %s\n", linein);
 	}
 	else {
-		*(ptr+1) = '\0';	/* ensure null termination */
+		*(end_ptr+1) = '\0';	/* ensure null termination */
 		
-		/* now see if it's in the linked list */
-		curr = master->head;
-		if(curr == NULL) {
-			/* special case, 1st node */
-			if((master->head = allocnode(st_ptr, mandatory)) == NULL) {
-				retval = RETVAL_ERROR;
-			}
-			else {
-				master->nritems++;
-			}
+		if((ptr = malloc(sizeof(List))) == NULL) {
+			error_log(ERRLOG_REPORT, "Out of Memory, aborting");
+			retval = RETVAL_ERROR;
 		}
-		else {	
-			do {
-				/* do a prelim 1st test so that we don't have to always call strcmp() */
-				/* can't use first char, since its a <, so try second character */
-				/* hopefully this will speed things up a bit. */
-				match = (st_ptr[1] != curr->msgnr[1]) ? 1 : strcmp(st_ptr, curr->msgnr);
-				prev = curr;
-				curr = curr->next;			
-			}
-			while(match != 0 && curr != NULL);
-			if(match == 0) {
-				if(removed != NULL) {
-					(*removed)++;
-				}
-				prev->mandatory = (char) mandatory;			
-				/* this is needed in case we are in mandatory list */
-				/* and have a dupe of one in optional list */
+		else if((ptr->msgnr = calloc(strlen(st_ptr)+1, sizeof(char))) == NULL) {
+			error_log(ERRLOG_REPORT, "Out of Memory, aborting");
+			free(ptr);
+			retval = RETVAL_ERROR;
+		}
+		else {
+			strcpy(ptr->msgnr, st_ptr);
+			ptr->next = NULL;
+			ptr->mandatory = (char) mandatory;
+
+			/* now put on list */
+			if( curr == NULL) {
+				/* first node */
+				master->head = curr = ptr;
 			}
 			else {
-				/* reached end of list */
-				if((prev->next = allocnode(st_ptr, mandatory)) == NULL) {
-					retval = RETVAL_ERROR;
-				}
-				else {
-					master->nritems++;
-				}
+				curr->next = ptr;
+				curr = ptr;
 			}
+			master->nritems++;
 		}
 	}
 	return retval;
 }
-/*-----------------------------------------------------------------------*/
-PList allocnode(char *linein, int mandatory) {
-	/* if allocate memory here, must free in free_one_node */
-	PList ptr;
-
-	if((ptr = malloc(sizeof(List))) == NULL) {
-		error_log(ERRLOG_REPORT, "Out of Memory, aborting");
-	}
-	else if((ptr->msgnr = calloc(strlen(linein)+1, sizeof(char))) == NULL) {
-		error_log(ERRLOG_REPORT, "Out of Memory, aborting");
-		free(ptr);
-	}
-	else {
-		strcpy(ptr->msgnr, linein);
-		ptr->next = NULL;
-		ptr->mandatory = (char) mandatory;
-	}
-	return ptr;
-}
 /*------------------------------------------------------------------------*/
 void free_one_node(PList node) {
 
@@ -688,7 +649,7 @@ void free_one_node(PList node) {
 	free(node);
 }
 /*----------------------------------------------------------*/
-int do_innbatch(int articleCount, char *batchfile) {
+int do_innbatch(PMaster master) {
 	/* build batch file that contains article listing */
 	/* needed by innxmit */
 	/* this is done by searching thru MSGDIR to find files */
@@ -700,11 +661,8 @@ int do_innbatch(int articleCount, char *batchfile) {
 	DIR *dptr;
 	struct dirent *entry;
 
-#ifdef DEBUG2
-	do_debug("BATCH FILE: %s\n", batchfile);
-#endif
-	if((fptr = fopen(batchfile, "w")) == NULL) {
-		MyPerror(batchfile);
+	if((fptr = fopen(master->batchfile, "w")) == NULL) {
+		MyPerror(master->batchfile);
 		retval = RETVAL_ERROR;
 	}
 	else if((dptr = opendir(full_path(FP_GET, FP_MSGDIR, ""))) == NULL) {
@@ -713,7 +671,7 @@ int do_innbatch(int articleCount, char *batchfile) {
 		fclose(fptr);
 	}
 	else {
-		sprintf(tmp, "-%d",articleCount);	/* this will be string we search for */
+		sprintf(tmp, "-%d",master->nritems);	/* this will be string we search for */
 		/* look for entries which have our file name ...-xxxxx where xxxx is articleCount */
 		while((entry = readdir(dptr)) != NULL) {
 			if(strstr(entry->d_name, tmp) != NULL) {
@@ -726,7 +684,8 @@ int do_innbatch(int articleCount, char *batchfile) {
 	return retval;
 }
 /*----------------------------------------------------------*/
-int do_rnewsbatch(int articleCount, char *batchfile, long max_file_size) {
+int do_rnewsbatch(PMaster master) {
+
 	/* build rnews formated file of articles */
 	/* this is done by searching thru MSGDIR to find files */
 	/* which match our naming convention */
@@ -735,7 +694,8 @@ int do_rnewsbatch(int articleCount, char *batchfile, long max_file_size) {
 
 	int i, batchnr = 0, retval = RETVAL_OK;
 	FILE *fptr = NULL, *fpin;
-	char tmp[20], *tptr, buf[MAXLINLEN];
+	const char *tptr;
+	char tmp[20], buf[MAXLINLEN];
 	DIR *dptr;
 	struct dirent *entry;
 	struct stat sbuf;
@@ -747,7 +707,7 @@ int do_rnewsbatch(int articleCount, char *batchfile, long max_file_size) {
 		fclose(fptr);
 	}
 	else {
-		sprintf(tmp, "-%d",articleCount);	/* this will be string we search for */
+		sprintf(tmp, "-%d",master->nritems);	/* this will be string we search for */
 		/* look for entries which have our file name ...-xxxxx where xxxx is articleCount */
 		while(retval == RETVAL_OK && (entry = readdir(dptr))) {
 			if(strstr(entry->d_name, tmp) != NULL) {
@@ -768,10 +728,10 @@ int do_rnewsbatch(int articleCount, char *batchfile, long max_file_size) {
 						}
 						/* have to open file */
 						if(batchnr == 0) {
-							strcpy(buf, batchfile);
+							strcpy(buf, master->batchfile);
 						}
 						else {
-							sprintf(buf, "%s%d", batchfile, batchnr);
+							sprintf(buf, "%s%d", master->batchfile, batchnr);
 						}
 #ifdef DEBUG2
 						do_debug("BATCH FILE: %s\n", buf);
@@ -783,7 +743,7 @@ int do_rnewsbatch(int articleCount, char *batchfile, long max_file_size) {
 					}
 					if(retval == RETVAL_OK) {
 						/* first put #! rnews size */
-						fprintf(fptr, "#! rnews %ld\n", sbuf.st_size);
+						fprintf(fptr, "#! rnews %ld\n", (long) sbuf.st_size);
 		
 						/* use fread/fwrite in case lines are longer than MAXLINLEN */
 						while((i = fread(buf, 1, MAXLINLEN, fpin)) > 0) {
@@ -795,7 +755,7 @@ int do_rnewsbatch(int articleCount, char *batchfile, long max_file_size) {
 						/* keep track of current file size, we can ignore the #! rnews */
 						/* size, since it adds so little to the overall size */
 					}
-					if(max_file_size > 0L && cursize > max_file_size) {
+					if(master->rnews_size > 0L && cursize > master->rnews_size) {
 						/* reached file size length */
 						cursize = 0L;	/* this will force a close and open on next article */
 					}
@@ -810,7 +770,8 @@ int do_rnewsbatch(int articleCount, char *batchfile, long max_file_size) {
 /*-----------------------------------------------------------------------------------------*/
 int get_one_article(PMaster master, int logcount) {
 
-	char buf[MAXLINLEN+1], *fname;
+	char buf[MAXLINLEN+1];
+	const char *fname;
 	int retval;
 	FILE *fptr;
 
@@ -818,7 +779,7 @@ int get_one_article(PMaster master, int logcount) {
 
 	/* build command to get article header*/
 	sprintf(buf, "article %s\r\n", (master->curr)->msgnr);
-	switch(send_a_command(master->sockfd, buf, 220)) {
+	switch(send_command(master, buf, NULL, 220)) {
 	  case RETVAL_OK:
 		if(master->MultiFile == TRUE) {
 			/* open file */
@@ -899,39 +860,6 @@ int get_a_chunk(int sockfd, FILE *fptr) {
 	}
 	return retval;
 }
-/*---------------------------------------------------------------------------*/
-int send_a_command(int sockfd, char *cmd, int good_response) {
-
-	char *inbuf;
-	int response, retval;
-#ifdef TIMER
-	int len;
-#endif
-	retval = RETVAL_OK;
-
-#ifdef DEBUG2
-	do_debug("\nSENDING REQUEST\n%s",cmd);
-#endif
- 	sputline(sockfd,cmd);
-#ifndef TIMER
- 	if(sgetline(sockfd,&inbuf) < 0) {
-#else
-	if((len = sgetline(sockfd, &inbuf)) < 0) {
-#endif
-		retval = RETVAL_ERROR;
-	}
-	else {
-#ifdef TIMER
-		TimerFunc(TIMER_ADDBYTES, len);
-#endif
-		number(inbuf, &response);
-		if (response != good_response) {
-			error_log(ERRLOG_REPORT,"*** Unexpected response to command: %s\n%s\n", cmd,inbuf);
-			retval = RETVAL_UNEXPECTEDANS;
-		}
-	}
-	return retval;
-}		
 /*-----------------------------------------------------------------------------*/
 int restart_yn(PMaster master) {
 	/* the restart file consists of 1 byte for each article prior to its */
@@ -942,10 +870,11 @@ int restart_yn(PMaster master) {
 	struct stat buf;
 	int i, retval = RESTART_NO;
 	FILE *fp;
-	char msgnr[MAXLINLEN];
-	char optional;
+	char msgnr[MAXLINLEN], optional;
+	const char *fre, *fso;
 
-	if(stat(full_path(FP_GET, FP_TMPDIR, N_RESTART), &buf) == 0) {
+	fre = full_path(FP_GET, FP_TMPDIR, N_RESTART);
+	if(stat(fre, &buf) == 0) {
 		/* If suck.restart exists something went wrong */
 		retval = RESTART_YES;	
 		master->itemon = (buf.st_size == 0) ? 0 : buf.st_size - 1;
@@ -954,14 +883,16 @@ int restart_yn(PMaster master) {
 
 		i = RETVAL_OK;
 
-		if((fp = fopen(full_path(FP_GET, FP_TMPDIR, N_SORTED), "r")) != NULL) {
+		fso = full_path(FP_GET, FP_TMPDIR, N_SORTED);
+		if((fp = fopen(fso, "r")) != NULL) {
 			while(i == RETVAL_OK && fscanf(fp, "%s %c", msgnr, &optional) == 2) {
-				i = build_list(master, msgnr, NULL, optional);
+				i = allocnode(master, msgnr, optional);
 			}
 			fclose(fp);
 		}
 		else {
 			i = RETVAL_ERROR;
+			error_log(ERRLOG_REPORT, "Whoops, %s exists, but %s doesn't, aborting\n", fre, fso);
 		}
 		if(i == RETVAL_ERROR) {
 			retval = RESTART_ERROR;
@@ -971,10 +902,342 @@ int restart_yn(PMaster master) {
 }
 /*-----------------------------------------------------------------*/
 #ifdef MYSIGNAL
-void sighandler(int what) {
+RETSIGTYPE sighandler(int what) {
 
 	error_log(ERRLOG_REPORT, "Signal received, will finish downloading article.\n");
 	GotSignal = TRUE;
 
 }
 #endif
+/*----------------------------------------------------------------*/
+int send_command(PMaster master, const char *cmd, char **ret_response, int good_response) {
+	/* this is needed so can do user authorization */
+
+	int len, retval = RETVAL_OK, nr;
+	char buf[256], *resp;
+
+#ifdef DEBUG2
+	do_debug("sending command: %s", cmd);
+#endif
+	sputline(master->sockfd, cmd);
+	len = sgetline(master->sockfd, &resp);	
+	if( len < 0) {	
+		retval = RETVAL_ERROR;		  	
+	}					
+	else {
+#ifdef DEBUG2
+		do_debug("got answer: %s", resp);
+#endif
+#ifdef TIMER
+		TimerFunc(TIMER_ADDBYTES, len);
+#endif
+		number(resp, &nr);
+		if(nr == 480 ) { /* we must do authorization */
+			sprintf(buf, "AUTHINFO USER %s\r\n", master->userid);
+#ifdef DEBUG2
+			do_debug("sending command: %s", buf);
+#endif
+			sputline(master->sockfd, buf);
+			len = sgetline(master->sockfd, &resp);
+			if( len < 0) {	
+				retval = RETVAL_ERROR;		  	
+			}					
+			else {
+#ifdef DEBUG2
+				do_debug("got answer: %s", resp);
+#endif
+#ifdef TIMER
+				TimerFunc(TIMER_ADDBYTES, len);
+#endif
+				number(resp, &nr);
+				if(nr != 381) {
+					error_log(ERRLOG_REPORT, "Weird Response to Authorization: %s", resp);
+					retval = RETVAL_NOAUTH;
+				}
+				else {
+					sprintf(buf, "AUTHINFO PASS %s\r\n", master->passwd);
+					sputline(master->sockfd, buf);
+#ifdef DEBUG2
+					do_debug("sending command: %s", buf);
+#endif
+					len = sgetline(master->sockfd, &resp);
+					if(len < 0) {	
+						retval = RETVAL_ERROR;		  	
+					}					
+					else {
+#ifdef DEBUG2
+						do_debug("got answer: %s", resp);
+#endif
+#ifdef TIMER
+						TimerFunc(TIMER_ADDBYTES, len);
+#endif
+						number(resp, &nr);
+						switch(nr) {
+						  case 281: /* bingo resend original command*/
+							sputline(master->sockfd, cmd);
+#ifdef DEBUG2
+							do_debug("sending command: %s", cmd);
+#endif
+							len = sgetline(master->sockfd, &resp);
+							if( len < 0) {	
+								retval = RETVAL_ERROR;		  	
+							}
+							else {
+								number(resp,&nr);
+#ifdef DEBUG2
+								do_debug("got answer: %s", resp);
+#endif
+#ifdef TIMER
+								TimerFunc(TIMER_ADDBYTES, len);
+#endif
+							}
+							break;
+						  case 502: /* permission denied */
+							retval = RETVAL_NOAUTH;
+							error_log(ERRLOG_REPORT, "Authorization Denied");
+							break;
+						  default: /* wacko error */
+							error_log(ERRLOG_REPORT, "Weird Response to Authorization: %s", resp);
+							retval = RETVAL_NOAUTH;
+							break;
+						}
+					}
+				}
+			}
+		}
+		if (good_response != 0 && nr != good_response) {
+			error_log(ERRLOG_REPORT,"*** Unexpected response to command: %s\n%s\n", cmd,resp);
+			retval = RETVAL_UNEXPECTEDANS;
+		}
+	}
+	if(ret_response != NULL) {
+		*ret_response = resp;
+	}
+	return retval;
+}
+/*------------------------------------------------------------------------------*/
+/* 1. move N_OLDRC to N_OLD_OLDRC					        */
+/* 2. move N_NEWRC to N_OLDRC							*/
+/* 3. rm N_SORTED								*/
+/* 4. rm N_SUPPLEMENTAL								*/
+/*------------------------------------------------------------------------------*/
+void do_cleanup() {
+	const char *oldptr;
+	char ptr[PATH_MAX+1];
+	strcpy(ptr,full_path(FP_GET, FP_DATADIR, N_OLDRC));	
+	/* must strcpy since full path overwrites itself everytime */
+	oldptr = full_path(FP_GET, FP_DATADIR, N_OLD_OLDRC);
+
+	if(rename(ptr, oldptr) != 0) {
+		MyPerror("Moving newsrc to backup:");
+	}
+	else if(rename(full_path(FP_GET, FP_TMPDIR, N_NEWRC), ptr) != 0) {
+		MyPerror("Moving newrc to newsrc:");
+	}
+	else if(unlink(full_path(FP_GET, FP_TMPDIR, N_SORTED)) != 0 && errno != ENOENT) {
+		/* ENOENT is not an error since this file may not always exist */
+		/* and it won't if there weren't any articles */
+		MyPerror("Removing Sorted file:");
+	}
+	else if(unlink(full_path(FP_GET, FP_DATADIR, N_SUPPLEMENTAL)) != 0 && errno != ENOENT) {
+		/* ENOENT is not an error since this file may not always exist */
+		MyPerror("Removing Supplemental Message file:");
+	}
+}
+/*---------------------------------------------------------------------------------------*/
+int scan_args(PMaster master, int argc, char *argv[]) {
+
+	int loop, retval = RETVAL_OK;
+	const char *ptr;
+	
+	for(loop=0;loop<argc && retval == RETVAL_OK;loop++) {
+#ifdef DEBUG2
+		do_debug("Checking arg #%d-%d: '%s'\n", loop, argc, argv[loop]);
+#endif
+		/* check for valid args format */
+		if(argv[loop][0] == '-') {
+			if(argv[loop][1] == 'b' || argv[loop][1] == 'd') {
+				if(argv[loop][3] != '\0') {
+					retval = RETVAL_ERROR;
+				}
+			}
+			else if( argv[loop][2] != '\0') {
+				retval = RETVAL_ERROR;
+			}
+		}
+		else if( argv[loop][0] != FILE_CHAR ){
+			/* we just ignore the file_char arg, it is processed elswhere */
+			retval = RETVAL_ERROR;
+		}
+		if(retval != RETVAL_OK) {
+			error_log(ERRLOG_REPORT, "Invalid argument: %s\n", argv[loop]);
+		}
+		else if(argv[loop][0] != FILE_CHAR) {
+			switch(argv[loop][1]) {
+			  case 'a':	/* if we have downloaded at least one article, then batch up */
+					/* even on errors */
+				master->always_batch = TRUE;
+				break;
+			  case 'h':
+				master->host = (loop+1 == argc) ? getenv("NNTPSERVER") : argv[++loop];
+				break;	
+			  case 'm':
+				master->MultiFile = TRUE;
+				break;
+			  case 'r':
+				if(loop+1 == argc) {
+					error_log(ERRLOG_REPORT, "No rnews batch file size provided\n");
+					retval = RETVAL_ERROR;
+				}
+				else {
+					master->rnews_size = atol(argv[++loop]);
+				}
+				break;
+			  case 'p':
+				if(loop+1 == argc) {
+					error_log(ERRLOG_REPORT, "No postfix provided\n");
+					retval = RETVAL_ERROR;
+				}
+				else {
+					full_path(FP_SET_POSTFIX, FP_NONE, argv[++loop]);
+				}
+				break;
+			  case 'd':	/* change one of the directory names */
+				if(loop+1 == argc) { 
+					error_log(ERRLOG_REPORT, "No directory name provided\n");
+					retval = RETVAL_ERROR;
+				}
+				else {
+					ptr = NULL;
+					switch (argv[loop][2]) {
+					  case 't':
+						ptr = full_path(FP_SET, FP_TMPDIR, argv[++loop]);
+						break;
+					  case 'd':
+						ptr = full_path(FP_SET, FP_DATADIR, argv[++loop]);
+						break;
+					  case 'm':
+						ptr = full_path(FP_SET, FP_MSGDIR, argv[++loop]);
+						break;
+					}
+					if(ptr == NULL) { 
+						error_log(ERRLOG_REPORT, "Invalid directory\n");
+						retval = RETVAL_ERROR;
+						break;
+					}
+				}
+				break;		 
+			  case 'b':	/* batch file implies MultiFile Mode */
+				switch (argv[loop][2]) {
+				  case 'i':
+					master->batch = BATCH_INNXMIT;
+					master->MultiFile = TRUE;		
+					break;
+				  case 'r':
+					master->batch = BATCH_RNEWS;
+					master->MultiFile = TRUE;
+					break;
+				  default:
+					error_log(ERRLOG_REPORT, "Invalid Batch arg: %s\n", argv[loop]);
+					retval = RETVAL_ERROR;
+					break;
+			  	}
+				if(retval == RETVAL_OK && loop+1 == argc) {
+					error_log(ERRLOG_REPORT, "No Batch file name provided\n");
+					retval = RETVAL_ERROR;
+				}
+				else {
+					master->batchfile = argv[++loop];
+				}
+				break;
+			  case 'e':	/* use default error log path */
+				error_log(ERRLOG_SET_FILE, ERROR_LOG);
+				break;
+			  case 'E':	/* error log path */
+				if(loop+1 == argc) { 
+					error_log(ERRLOG_REPORT, "No Error Log name provided\n");
+					retval = RETVAL_ERROR;
+				}
+				else {
+					error_log(ERRLOG_SET_FILE, argv[++loop]);
+				}
+				break;
+			  case 's':	/* use default status log name */
+				master->status_file_name = STATUS_LOG;
+				break;
+			  case 'S':	/* status log path */
+				if(loop+1 == argc) {
+					error_log(ERRLOG_REPORT, "No Status Log name provided\n");
+					retval = RETVAL_ERROR;
+				}
+				else {
+					master->status_file_name = argv[++loop];
+				}
+				break;
+			  case 'V': 	/* show version number */
+			  	error_log(ERRLOG_REPORT, "Suck version %d.%d.%d\n",SUCKVER, SUCKVER_MINOR, SUCKVER_PATCH);
+				retval = RETVAL_VERNR;	/* so we don't do anything else */
+			  	break;
+			  case 'K':
+				master->do_killfile = FALSE;
+				break;
+			  case 'H':
+				master->do_chkhistory = FALSE;
+				break;
+			  case 'U':	/* userid */
+				if(loop+1 == argc) {
+					error_log(ERRLOG_REPORT, "No Userid provided\n");
+					retval = RETVAL_ERROR;
+				}
+				else {
+					master->userid = argv[++loop];
+				}
+				break;
+			  case 'P':	/* passwd */
+				if(loop+1 == argc) {
+					error_log(ERRLOG_REPORT, "No Passwd provided\n");
+					retval = RETVAL_ERROR;
+				}
+				else {
+					master->passwd = argv[++loop];
+				}
+				break;
+			  case 'c': 	/* cleanup option */
+				master->cleanup = TRUE;
+				break;
+			  case 'M': /* mode reader */
+				master->do_modereader = TRUE;
+				break;
+			  case 'N':	/* override default portnr */
+				if(loop+1 == argc) {
+					error_log(ERRLOG_REPORT, "No Port Number provided\n");
+					retval = RETVAL_ERROR;
+				}
+				else {
+					master->portnr = atoi(argv[++loop]);
+				}
+				break;
+			  case 'W':	/* wait (sleep) x seconds between every y articles */
+				if(loop+2 == argc) {
+					error_log(ERRLOG_REPORT, "Invalid pause arguments\n");
+					retval = RETVAL_ERROR;
+				}
+				else {
+					master->pause_time = atoi(argv[++loop]);
+					master->pause_nrmsgs = atoi(argv[++loop]); 
+				}
+				break;
+			  case 'k':	/* microsoft news xhdr 224 kludge */
+				master->ms_kludge = TRUE;
+				break;
+			  default:
+				error_log(ERRLOG_REPORT, "Invalid argument: %s\n", argv[loop]);
+				retval = RETVAL_ERROR;
+				break;
+			}			
+		}
+	}
+
+	return retval;
+
+}
